@@ -18,6 +18,8 @@ from sklearn.dummy import DummyClassifier
 from . import metrics
 from .hpo import find_best_params
 from .utils import print_progress_bar
+from .metrics import loss_threshold
+
 
 warnings.filterwarnings(action='ignore', category=ConvergenceWarning)
 
@@ -53,7 +55,7 @@ class Classifier:
 
         self.data = data.reset_index(drop=True)
         self.X = data.drop(columns=self.labels_col).values
-        self.categories = np.unique(self.y).tolist()
+        self.categories = list(np.unique(self.y))
         self.N = len(data)
         self.cv = cv
 
@@ -136,10 +138,11 @@ class Classifier:
         start = time.time()
 
         self.logger.info("Estimating instance performance...")
+        if not hyper_param_optm:
+            self.logger.debug(f"Training classifier with default parameters {kwargs}")
         for i in range(n_iter):
             k = 0
-            if not hyper_param_optm:
-                self.logger.debug(f"Training classifier with default parameters {kwargs}")
+            accuracy = 0
             for train_index, test_index in kf.split(self.X, self.y):
                 k += 1
                 self.logger.info(f"Evaluating testing fold #{k}")
@@ -164,11 +167,19 @@ class Classifier:
                 y_pred = clf.predict_proba(X_test)
                 score[test_index, i], proba[test_index, i] = self.score(metric=metric, y_true=self.y[test_index],
                                                                         y_pred=y_pred, classes_order=clf.classes_)
-                self.logger.info(f"Test fold mean accuracy: {clf.score(X_test, self.y[test_index])}")
+
+                fold_accuracy = clf.score(X_test, self.y[test_index])
+                accuracy += fold_accuracy
+                self.logger.info(f"Test fold mean accuracy: {fold_accuracy}")
             print_progress_bar(i + 1, n_iter, prefix='Progress', suffix=f'complete (CV {i + 1}/{n_iter})', length=30)
 
+            thres = loss_threshold(n_classes=len(self.categories), metric=metric)
+            self.logger.info(f"Mean accuracy on test instances (iteration #{i+1}): {round(accuracy / k, 4)}")
+            self.logger.debug(f"Proximate conversion of metric '{metric}' to accuracy (mean): "
+                              f"{np.mean(score <= thres)}")
+
         end = time.time()
-        self.logger.debug(f"Total elapsed time: {(end - start):.2f}")
+        self.logger.debug(f"Elapsed time: {(end - start):.2f}")
 
         return score.mean(axis=1), proba.mean(axis=1)
 
