@@ -2,6 +2,7 @@ import argparse
 import logging
 import re
 import shutil
+import time
 from importlib import import_module
 from inspect import signature, Parameter
 from logging.handlers import RotatingFileHandler
@@ -9,17 +10,22 @@ from pathlib import Path
 
 import pandas as pd
 from pyispace.example import save_opts
+from pyispace.trace import trace_build_wrapper
+from pyispace.utils import save_footprint
 
 from . import integrator, log_file, formatter
 from .context import Configuration
 from .feature_selection import featfilt
 from .hpo import set_hyperopt_progressbar
+from .metrics import loss_threshold
 from .visualization import Demo, App
 
+
 _conf_file = 'config.yaml'
 
 
 def main(args):
+    start = time.time()
     logger = logging.getLogger(__name__)
     _my_path = Path().absolute()
 
@@ -45,7 +51,7 @@ def main(args):
 
         file_path = Path(conf.get('datafile'))
         if file_path.is_file():
-            logger.info("Reading input dataset: {0}".format(file_path))
+            logger.info("Reading input dataset: '{0}'".format(file_path))
             df_dataset = pd.read_csv(file_path)
         else:
             logger.error("Invalid datafile '{0}'".format(file_path))
@@ -58,25 +64,50 @@ def main(args):
             logger.info("Building metadata.")
             df_metadata, df_ih = integrator.build_metadata(data=df_dataset, return_ih=True,
                                                            verbose=args.verbose, **kwargs)
+        else:
+            df_metadata = pd.read_csv(rootdir_path / 'metadata.csv', index_col='instances')
+            df_ih = pd.read_csv(rootdir_path / 'ih.csv', index_col='instances')
 
         if args.isa:
             if conf.get('feat_select'):
-                logger.info("Feature selection on")
-                if 'df_metadata' not in locals():
-                    df_metadata = pd.read_csv(rootdir_path / 'metadata.csv', index_col='instances')
-                df_metadata.to_csv(rootdir_path / 'metadata_original.csv')
+                n_feat_cols = len(df_metadata.filter(regex='^feature_').columns)
+                if n_feat_cols > conf.get('max_n_features'):
+                    logger.info("Feature selection on")
+                    if 'df_metadata' not in locals():
+                        df_metadata = pd.read_csv(rootdir_path / 'metadata.csv', index_col='instances')
 
-                sig = signature(featfilt)
-                param_dict = {param.name: kwargs[param.name] for param in sig.parameters.values()
-                              if param.kind == param.POSITIONAL_OR_KEYWORD and param.default != Parameter.empty and
-                              param.name in kwargs}
-                selected, df_metadata = featfilt(df_metadata, **param_dict)
-                logger.info("Selected features: {0}".format(selected))
+                    df_metadata.to_csv(rootdir_path / 'metadata_original.csv')
+                    sig = signature(featfilt)
+                    param_dict = {param.name: kwargs[param.name] for param in sig.parameters.values()
+                                  if param.kind == param.POSITIONAL_OR_KEYWORD and param.default != Parameter.empty and
+                                  param.name in kwargs}
+                    selected, df_metadata = featfilt(df_metadata, **param_dict)
+                    logger.info("Selected features: {0}".format(selected))
+                    df_metadata.to_csv(rootdir_path / 'metadata.csv')
+                else:
+                    logger.info("Skipping feature selection: "
+                                "number of features already satisfied "
+                                f"({n_feat_cols} <= max_n_features ({conf.get('max_n_features')}))")
 
             isa_engine = str.lower(conf.get('isa_engine'))
             logger.info(f"Running Instance Space Analysis with {repr(isa_engine)} engine.")
             if isa_engine == 'python':
-                integrator.run_isa(rootdir=rootdir_path)
+                # changes ISA 'perf':'epsilon' option
+                epsilon = conf.get('perf_threshold')
+                if epsilon == 'auto':
+                    n_classes = df_dataset.iloc[:, -1].nunique()
+                    epsilon = loss_threshold(n_classes, metric=conf.get('metric'))
+                other = {'perf': {'epsilon': epsilon}}
+
+                model = integrator.run_isa(rootdir=rootdir_path, settings=other)
+
+                threshold = conf.get('ih_threshold')
+                pi = conf.get('ih_purity')
+                logger.info("Calculating instance hardness footprint area")
+                logger.info(f"An instance is easy if its IH-value <= {threshold}")
+                Ybin = df_ih.values[:, 0] <= threshold
+                ih_fp = trace_build_wrapper(model.pilot.Z, Ybin, pi)
+                save_footprint(ih_fp, rootdir_path, 'instance_hardness')
             elif isa_engine == 'matlab':
                 _ = integrator.run_matilda(metadata=df_metadata, rootdir=conf.get('rootdir'),
                                            matildadir=conf.get('matildadir'))
@@ -87,9 +118,7 @@ def main(args):
 
         if args.app:
             logging.getLogger().setLevel(logging.WARNING)
-            if not args.meta:
-                df_metadata = pd.read_csv(rootdir_path / 'metadata.csv', index_col='instances')
-                df_ih = pd.read_csv(rootdir_path / 'ih.csv', index_col='instances')
+
             df_metadata = df_ih.join(df_metadata, how='right')
             df_is = pd.read_csv(rootdir_path / 'coordinates.csv', index_col='Row')
             df_foot_perf = pd.read_csv(rootdir_path / 'footprint_performance.csv', index_col='Row')
@@ -117,6 +146,13 @@ def main(args):
             app = App(df_dataset, df_metadata, df_is, df_footprint, df_foot_perf)
             app.show(port=5001, websocket_origin=['127.0.0.1:5001', 'localhost:5001'])
 
+        end = time.time()
+        elapsed_time = end - start
+        if elapsed_time < 60:
+            logger.info(f"Total elapsed time: {elapsed_time:.1f}s")
+        else:
+            elapsed_time /= 60
+            logger.info(f"Total elapsed time: {elapsed_time:.1f}min")
         logger.info("Instance Hardness analysis finished.")
 
 
