@@ -1,20 +1,22 @@
+import glob
+import json
+import logging
 import os
 import re
-import json
-import warnings
-import logging
 import shutil
-import glob
+import warnings
+from importlib import import_module
+from inspect import signature, Parameter
+from pathlib import Path
+
 import pandas as pd
 from pyispace import train_is
 from pyispace.utils import scriptcsv
-from pathlib import Path
-from importlib import import_module
-from inspect import signature, Parameter
-from .measures import Measures
+
 from .classification import Classifier
-from .utils import get_param_names
 from .feature_selection import featfilt
+from .measures import Measures
+from .utils import get_param_names
 
 
 metadata_file = 'metadata.csv'
@@ -111,7 +113,14 @@ def run_matilda_module(rootdir):
         obj.terminate()
 
 
-def run_isa(rootdir: Path):
+def update_options(d1: dict, d2: dict):
+    d1 = d1.copy()
+    for k in d1.keys():
+        d1[k] = {**d1[k], **d2.get(k, dict())}
+    return d1
+
+
+def run_isa(rootdir: Path, settings=None, save_output=True):
     if not rootdir.exists():
         raise NotADirectoryError(f"Invalid directory {repr(repr(rootdir))}.")
 
@@ -128,9 +137,13 @@ def run_isa(rootdir: Path):
     else:
         raise FileNotFoundError(f"File 'metadata.csv' not found in specified path {repr(str(rootdir))}.")
 
+    opts = update_options(opts, settings)
+
     logging.getLogger('pyispace').setLevel(logging.DEBUG)
     out = train_is(metadata, opts)
-    scriptcsv(out, rootdir)
+    if save_output:
+        scriptcsv(out, rootdir)
+    return out
 
 
 def clear_cache():
