@@ -311,7 +311,7 @@ class Measures:
             aux = (nn == label)[::-1]
             ind_nn_max = len(aux) - np.argmax(aux) - 1
             LSR[i] = min(1.0, distances[i, np.argmax(nn != label)] / distances[i, ind_nn_max])
-        return LSR
+        return 1 - LSR
 
     def harmfulness(self, distance='gower'):
         y = self.y.values.copy()
