@@ -1,19 +1,21 @@
 import io
 import logging
-import pandas as pd
-import numpy as np
-import panel as pn
-import holoviews as hv
 from pathlib import Path
-from holoviews import opts, dim, streams
-from holoviews.plotting.links import DataLink
+
+import holoviews as hv
+import numpy as np
+import pandas as pd
+import panel as pn
 from bokeh.models import HoverTool
 from bokeh.themes.theme import Theme
+from holoviews import opts, dim, streams
+from holoviews.plotting.links import DataLink
 from shapely.geometry import Point, Polygon, MultiPolygon
-from shapely.errors import TopologicalError
 from sklearn.decomposition import PCA
+
 from pyhard.utils import reduce_dim
 
+
 dark_color = '#292929'
 light_color = '#ffffff'
 # footprint_colors = {'good': '#3F81BC', 'bad': 'gray', 'best': 'green'}
@@ -179,7 +181,9 @@ class App:
         self.w_color_range = pn.widgets.RangeSlider(start=0, end=20, value=(0, 5), step=0.5)
         self.w_checkbox = pn.widgets.Checkbox(name='manual colorbar range', value=False)
         self.w_footprint_on = pn.widgets.Checkbox(name='draw footprint area', value=True)
-        self.w_footprint_algo = pn.widgets.Select(options=df_footprint.index.unique(level='algo').to_list(), value='')
+
+        val = 'instance_hardness' if 'instance_hardness' in df_footprint.index else ''
+        self.w_footprint_algo = pn.widgets.Select(options=df_footprint.index.unique(level='algo').to_list(), value=val)
 
         self.bbox = None
         self.cmap = 'coolwarm'
